% SESION 9
% - Interpolacion y aproximacion de datos (y funciones)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Interpolacin polinomial  (luego describiremos mas el  comando polyfit)
%
x=linspace(0,2*pi,16); % nodos de interpolacion
y=2*sin(x);   % datos de interpolacion
n=length(x); % número de datos
p=polyfit (x, y, n-1);  % el resultado es un polinomio que se puede evaluar directamente con el comando polyval
xx=linspace(0,2*pi,50);  % vamos a dibujar el polinomio en mas puntos, si no se ve fatal
inte=polyval(p,xx);  
plot(x,y,'*;datos;',xx,inte,';polinomio interpolante;')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%interpolacion 1D con splines
%
% yi = interp1 (nodos, valores, xi, metodo, extrap, 'pp')
%Interpola los puntos (nodos, valores) según el método que se ponga como cuarto argumento (por defecto es lineal) y la salida 
%es el resultado del interpolante evaluado en los puntos de xi
%Tipos de metodo
% 'linear' spline lineal continuo, recta a trozos
% 'nearest' constante a trozos
% 'pchip' spline cubico de clase 1, interp de hermite. ¿como calcula los datos de la derivada en los nodos? 
% 'cubic' interpolacion cubica usando 4 nodos ????
% 'spline' spline cubico de clase 2 (natural o sujeto, dependiendo de cómo sea el vector de "valores"). Lo vemos mas abajo.
%
% Opciones de interp1: 'pp'. No requiere preestablecer la "xi" para representar el resultado. 
% La salida es ideal para representarla con el comando "ppval". Asi no hay que prefijar los nodos para la representacion grafica.

% calculamos los datos de interpolacion
nodos = [0:10];
valores = sin (2*pi*nodos/5);   

% nodos para pintar la funcion original
x = [0:0.05:10];
y = sin (2*pi*x/5);

% spline constante a trozos
cte = interp1 (nodos, valores, x, "nearest");     
figure(1);
plot (x, y, "r", x, cte, "b", nodos, valores, "r*");          
legend ("original", "constante");

% spline lineal
lin = interp1 (nodos, valores, x);  
figure(2);
plot (x, y, "r", x, lin, "b", nodos, valores, "r*");          
legend ("original", "lineal");        

% spline cubico de clase dos natural (se anulan las segundas derivadas en los extremos)
clase2 = interp1 (nodos, valores, x, "spline");
figure(3);
plot (x, y, "r", x, clase2, "b",  nodos, valores, "r*");          
legend ("original", "clase 2");

% 
cubico4 = interp1 (nodos, valores, x, "cubic");          
figure(4);
plot (x, y, "r", x, cubico4, "b", nodos, valores, "r*");          
legend ("original", "cubico4");

%          
clase1 = interp1 (nodos, valores, x, "pchip");          
figure(5);
plot (x, y, "r", x, clase1, "b", nodos, valores, "r*");          
legend ("original", "clase 1");

%
figure(6);
plot (x, y, "r", x, lin, "g", x, clase2, "b", x, clase1, "c", x, cte, "m", x, cubico4, "b",nodos, valores, "r*");          
legend ("original", "lineal", "clase 2", "clase 1", "cubico4", "constante");
%

% Con la opcion 'pp'
clase1pp = interp1 (nodos, valores, "cubic",'pp');   
figure(6);      
plot (x, y, "r", x, ppval(clase1pp,x), "b", nodos, valores, "r*");  % podemos cambiar los nodos "x" para dibujar la grafica del interpolante, sin necesidad de volver a calcularlo.  

%  "interp1" para los spline cubicos usa un comando de interpolación que se puede llamar directamente: "spline"
% Calcula el spline cubico de clase 2 para los nodos y valores dados
% pp = spline(nodos, valores)  -->  para evaluarlo con "ppval"
% y = spline(nodos, valores, x) --> evaluacion del spline en x
% Si nodos y valores tienen la misma longitud, calcula el spline cubico natural. 
% Si hay dos valores mas que nodos, hace el spline cubico sujeto de clase 2. Los datos para la primera derivada en el primer y ultimo nodo se toman del primer y ultimo dato del vector "valores".

spline_natural=spline(nodos, valores, x);
figure(1);
plot (x, y, "r", x, spline_natural, "b", nodos, valores, "r*");          

valores_sujeto=[-1,valores];
valores_sujeto=[valores_sujeto,-1];
spline_sujeto=spline(nodos, valores_sujeto, x);
figure(2);
plot (x, y, "r", x, spline_sujeto, "b", nodos, valores, "r*");          

% ¿funcionara igual con el comando interp1?

clase2_sujeto = interp1 (nodos, valores_sujeto, x, "spline");
figure(3);
plot (x, y, "r", x, clase2_sujeto, "b",  nodos, valores, "r*");          
legend ("original", "clase 2_sujeto");



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Regresion lineal para ajustar una nube de puntos

% creamos un nube de datos a mano, que se parezca a y =x-3; le aadimos algo de aleatoriedad;
x=0:0.02:2;
y=3-x+(randn(size(x)))/4;

%regresion lineal
p=polyfit (x, y, n=1);
polyout(p,'x')
fit=polyval(p,x);
plot(x,y,'*;datos;',x,fit,';recta por minimos cuadrados;')

%En general
%[P, S] = polyfit (x, y, N)
%proporciona un polinomio de grado N obtenido por minimos cuadrados, minimiza la suma (y_i - p(x_i) )^2
%(x,y) son las coordenadas de los datos entradas y N es el grado del polinomio
%P es el polinomio (vector fila) que se obtiene por defecto
%S es una ESTRUCTURA compleja que contiene la siguiente informacion:
%-yf --> las evaluaciones del polinomio en los nodos, (que no se como extraerlas para utilizarlas despues)
%-R  --> factor de la descomposicion QR
%- X --> La matriz de Vandermonde asociada
%- df  --> grados de libertad
%-norm --> norma de los residuos
[P, S] = polyfit (x, y, 1)

%polyfit da el polinomio de interpolacion cuando N es mayor o igual al numero de datos.

%otro ejemplo: ajuste parabolico por minimos cuadrados
x=-2:0.02:2;
y=4-x.^2+(randn(size(x)))/3;
p=polyfit (x, y, n=2);
polyout(p,'x')
fit=polyval(p,x);
plot(x,y,'*;datos;',x,fit,';parabola  minimos cuadrados;')


